'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _writeJsonFile = require('write-json-file');

var _writeJsonFile2 = _interopRequireDefault(_writeJsonFile);

var _child_process = require('child_process');

var _child_process2 = _interopRequireDefault(_child_process);

var _colorLogger = require('color-logger');

var _colorLogger2 = _interopRequireDefault(_colorLogger);

var _readPkgUp = require('read-pkg-up');

var _readPkgUp2 = _interopRequireDefault(_readPkgUp);

var _Event = require('./Event');

var _Event2 = _interopRequireDefault(_Event);

var _schemaValidator = require('./util/schemaValidator');

var _schemaValidator2 = _interopRequireDefault(_schemaValidator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new _bluebird2.default(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return _bluebird2.default.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

const readFileAsync = _bluebird2.default.promisify(_fsExtra2.default.readFile);

let Npm = function () {
  /**
   * Create a new instance.
   *
   * @param {Object} config config object
   */
  function Npm(config) {
    _classCallCheck(this, Npm);

    this._debug = config.debug || false;

    _colorLogger2.default.debug = this._debug;
  }

  /**
   * Generates a plugin list.
   *
   * @param {String} manager  event context
   * @param {String} filePath path to new file
   *
   * @returns {Object} generated config
   */


  _createClass(Npm, [{
    key: 'generateConfig',
    value: (() => {
      var _ref = _asyncToGenerator(function* (manager, filePath) {
        _colorLogger2.default.i('generateConfig()');

        const configPath = filePath || '.plugged-in.json';

        try {
          const obj = yield (0, _readPkgUp2.default)();

          if (typeof obj.pkg['plugged-in'] === 'undefined') {
            throw new Error('package.json must have a `plugged-in` section definining the context');
          }

          const sysConfig = obj.pkg['plugged-in'];

          const configObj = {
            context: sysConfig.context,
            plugins: []
          };

          const modules = yield this.getModules();

          yield _bluebird2.default.mapSeries(modules, (() => {
            var _ref2 = _asyncToGenerator(function* (dir) {
              try {
                const plugPkg = yield Npm.findPackage(dir);

                if (plugPkg === null) {
                  return;
                }

                const config = plugPkg['plugged-in'];

                if (typeof config !== 'undefined') {
                  if (config.context === sysConfig.context) {
                    const name = plugPkg.name;

                    if (name === obj.pkg.name) {
                      return;
                    }

                    yield _schemaValidator2.default.validate('http://plugged-in.x-c-o-d-e.com/schema/configuration+v1#', config);

                    let plugin = { name: name };

                    delete config.context;

                    plugin = Object.assign(plugin, config);

                    configObj.plugins.push(plugin);
                  }
                }
              } catch (error) {
                _colorLogger2.default.e('process modules', error.message, dir);
              }
            });

            return function (_x3) {
              return _ref2.apply(this, arguments);
            };
          })());

          manager.addPlugins(configObj.plugins);

          const event = new _Event2.default({ name: 'generateConfig', data: configObj });

          manager.emit(event.name, event);

          yield (0, _writeJsonFile2.default)(configPath, event.data, { indent: 2 });

          return configObj;
        } catch (error) {
          _colorLogger2.default.e('generateConfig()', error.message);
        }

        return null;
      });

      function generateConfig(_x, _x2) {
        return _ref.apply(this, arguments);
      }

      return generateConfig;
    })()

    /**
     * Find package.json file.
     *
     * @param {String} [dir] optional directory to search
     *
     * @returns {Object} package
     */

  }, {
    key: 'executeShell',


    /**
     * Execute shell command.
     *
     * @param {String} command command line command
     *
     * @returns {String} output
     */
    value: (() => {
      var _ref3 = _asyncToGenerator(function* (command) {
        const bufferSize = 1024 * 500;

        const execAsync = _bluebird2.default.promisify(_child_process2.default.exec);

        try {
          return yield execAsync(command, { maxBuffer: bufferSize });
        } catch (error) {
          _colorLogger2.default.e(error.message.replace('Command failed: npm ls --parseable', '').replace('npm ERR! ', ''));

          return null;
        }
      });

      function executeShell(_x4) {
        return _ref3.apply(this, arguments);
      }

      return executeShell;
    })()

    /**
     * Get installed modules.
     *
     * @returns {String[]} list of all module directories
     *
     * @private
     */

  }, {
    key: 'getModules',
    value: (() => {
      var _ref4 = _asyncToGenerator(function* () {
        let modules = [];

        try {
          let result = yield this.executeShell('npm ls --parseable');

          modules = this._prepareModuleList(result);

          result = yield this.executeShell('npm ls -g --parseable');

          modules = modules.concat(this._prepareModuleList(result));
        } catch (error) {
          _colorLogger2.default.e(error);
        }

        return modules;
      });

      function getModules() {
        return _ref4.apply(this, arguments);
      }

      return getModules;
    })()

    /**
     * Prepares module list from shell output.
     *
     * @param {String} data shell output
     *
     * @returns {String[]} list of modules
     */

  }, {
    key: '_prepareModuleList',
    value: function _prepareModuleList(data) {
      const modules = [];

      data.split('\n').forEach(dir => {
        if (dir.trim() !== '') {
          modules.push(dir);
        }
      });

      return modules;
    }
  }], [{
    key: 'findPackage',
    value: (() => {
      var _ref5 = _asyncToGenerator(function* (dir) {
        const directory = dir || __dirname;
        let packageObj = null;

        try {
          const packageFilePath = _path2.default.join(directory, 'package.json');

          const exists = _fsExtra2.default.existsSync(packageFilePath);

          if (exists === false) {
            return packageObj;
          }

          const json = yield readFileAsync(packageFilePath, { encode: 'utf8' });

          packageObj = JSON.parse(json);
        } catch (error) {
          _colorLogger2.default.w('findPackage()', error.message);
        }

        return packageObj;
      });

      function findPackage(_x5) {
        return _ref5.apply(this, arguments);
      }

      return findPackage;
    })()
  }]);

  return Npm;
}();

exports.default = Npm;