'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _events = require('events');

var _colorLogger = require('color-logger');

var _colorLogger2 = _interopRequireDefault(_colorLogger);

var _Npm = require('./Npm');

var _Npm2 = _interopRequireDefault(_Npm);

var _Event = require('./Event');

var _Event2 = _interopRequireDefault(_Event);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new _bluebird2.default(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return _bluebird2.default.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

const fileSystem = _bluebird2.default.promisifyAll(_fs2.default);

let PluginManager = function (_EventEmitter) {
  _inherits(PluginManager, _EventEmitter);

  /**
   * Create instance.
   *
   * @param {Object}  [options]               options object
   * @param {Boolean} [options.debug]         debug flag
   * @param {String}  [options.context]       context name
   * @param {String}  [options.configFile]    set the config file path
   * @param {Boolean} [options.override=true] whether to override matching functions
   */
  function PluginManager() {
    let options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, PluginManager);

    _colorLogger2.default.i('new PluginManager()', options);

    var _this = _possibleConstructorReturn(this, (PluginManager.__proto__ || Object.getPrototypeOf(PluginManager)).call(this, options));

    _this._debug = options.debug || false;
    _this._override = options.override || true;
    _this._configFile = options.configFile || '.plugged-in.json';
    _this.context = options.context;

    _colorLogger2.default.debug = options.debug || false;
    return _this;
  }

  /**
   * Determine if running in debug mode.
   *
   * @returns {Boolean} true or false
   */


  _createClass(PluginManager, [{
    key: 'init',


    /**
     * Initialize with plugin property.
     *
     * @param {Object} plugin local event handlers
     *
     * @returns {PluginManager} this plugin
     */
    value: (() => {
      var _ref = _asyncToGenerator(function* () {
        let plugin = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

        _colorLogger2.default.i('PluginManager.init()');

        let data;

        try {
          const exists = fileSystem.existsSync(this._configFile);

          if (exists === true) {
            data = yield fileSystem.readFileAsync(this._configFile);

            data = JSON.parse(data);

            yield this.addPlugins(data.plugins);
          } else {
            _colorLogger2.default.d('.plugged-in.json does not exists');

            // find plugins
            const npm = new _Npm2.default({ debug: this._debug });

            data = yield npm.generateConfig(this);
          }

          if (typeof this.context === 'undefined') {
            this.context = data.context;
          }
        } catch (error) {
          _colorLogger2.default.e('PluginManager.init()', error.message);
        }

        // Add local plugin last so that it can override externals if necessary
        yield this.addPlugins([plugin]);

        this.dispatch('postInit', this);

        return this;
      });

      function init(_x2) {
        return _ref.apply(this, arguments);
      }

      return init;
    })()

    /**
     * Helper method to dispatch events.
     *
     * Note: The `data` parameter should be an object
     *
     * @param {String} eventName the event name
     * @param {Object} obj       the data object
     *
     * @returns {Object} the modified data object
     */

  }, {
    key: 'dispatch',
    value: (() => {
      var _ref2 = _asyncToGenerator(function* (eventName, obj) {
        if (typeof eventName === 'undefined') {
          throw new Error('Event name not provided');
        }

        let data = obj;

        if (typeof data === 'undefined') {
          data = {};
        }

        const event = new _Event2.default({ name: eventName, data: data });

        _colorLogger2.default.i(`Dispatching ${ event.name }...`);

        this.emit(eventName, event);

        return data;
      });

      function dispatch(_x4, _x5) {
        return _ref2.apply(this, arguments);
      }

      return dispatch;
    })()

    /**
     * Determines if a handler exists for an event.
     *
     * @param {String}          event   the event name
     * @param {Function|String} handler the function or its name
     *
     * @returns {Boolean} true if exists, otherwise false
     */

  }, {
    key: 'hasHandler',
    value: function hasHandler(event, handler) {
      const listeners = this.listeners(event).map(listener => listener.name);

      if (typeof handler === 'function') {
        return listeners.indexOf(handler.name) !== -1;
      }

      return listeners.indexOf(handler) !== -1;
    }

    /**
     * Determines if an event has any handlers.
     *
     * @param {String} event the event name
     *
     * @returns {Boolean} true or false
     */

  }, {
    key: 'hasHandlers',
    value: function hasHandlers(event) {
      if (typeof event === 'undefined') {
        throw new Error('Undefined event name');
      }

      return this.listeners(event).length > 0;
    }

    /**
     * Removes a handler for an event that matches the handler function name.
     *
     * @param {String}          event   the event name
     * @param {Function|String} handler the function or its name
     *
     * @returns {undefined}
     */

  }, {
    key: 'removeHandler',
    value: function removeHandler(event, handler) {
      let name = handler;

      if (typeof name === 'function') {
        name = handler.name;
      }

      this.listeners(event).forEach(listener => {
        if (listener.name === name) {
          this.removeListener(event, listener);
        }
      });
    }

    /**
     * Add local plugins.
     *
     * @param {Object[]} plugins list of plugins
     *
     * @returns {undefined}
     */

  }, {
    key: 'addPlugins',
    value: (() => {
      var _ref3 = _asyncToGenerator(function* (plugins) {
        var _this2 = this;

        if (Array.isArray(plugins) === false) {
          throw new Error('Plugins must be an array');
        }

        yield _bluebird2.default.mapSeries(plugins, (() => {
          var _ref4 = _asyncToGenerator(function* (plugin) {
            return _this2._processPlugin(plugin);
          });

          return function (_x7) {
            return _ref4.apply(this, arguments);
          };
        })());
      });

      function addPlugins(_x6) {
        return _ref3.apply(this, arguments);
      }

      return addPlugins;
    })()

    /**
     * Add plugin handlers.
     *
     * @param {Object} plugin the plugin
     *
     * @returns {undefined}
     *
     * @private
     */

  }, {
    key: '_processPlugin',
    value: (() => {
      var _ref5 = _asyncToGenerator(function* (plugin) {
        var _this3 = this;

        try {
          if (typeof plugin.provides === 'undefined') {
            return;
          }

          const pluginEvents = Object.keys(plugin.provides);

          yield _bluebird2.default.mapSeries(pluginEvents, (() => {
            var _ref6 = _asyncToGenerator(function* (eventName) {
              return _this3._processEvent(plugin, eventName);
            });

            return function (_x9) {
              return _ref6.apply(this, arguments);
            };
          })());
        } catch (error) {
          _colorLogger2.default.e(error.message);
        }
      });

      function _processPlugin(_x8) {
        return _ref5.apply(this, arguments);
      }

      return _processPlugin;
    })()

    /**
     * Add plugin event.
     *
     * @param {Object} plugin    the plugin
     * @param {String} eventName the event name
     *
     * @returns {undefined}
     *
     * @private
     */

  }, {
    key: '_processEvent',
    value: (() => {
      var _ref7 = _asyncToGenerator(function* (plugin, eventName) {
        var _this4 = this;

        try {
          let funcs = plugin.provides[eventName];

          if (Array.isArray(funcs) !== true) {
            funcs = [funcs];
          }

          yield _bluebird2.default.mapSeries(funcs, (() => {
            var _ref8 = _asyncToGenerator(function* (func) {
              return _this4._processHandler(plugin, eventName, func);
            });

            return function (_x12) {
              return _ref8.apply(this, arguments);
            };
          })());
        } catch (error) {
          _colorLogger2.default.e(error.message);
        }
      });

      function _processEvent(_x10, _x11) {
        return _ref7.apply(this, arguments);
      }

      return _processEvent;
    })()

    /**
     * Add plugin handler.
     *
     * @param {Object}   plugin    the plugin
     * @param {String}   eventName the event name
     * @param {Function} handler   the handler
     *
     * @returns {undefined}
     *
     * @private
     */

  }, {
    key: '_processHandler',
    value: (() => {
      var _ref9 = _asyncToGenerator(function* (plugin, eventName, handler) {
        try {
          let func = handler;

          if (typeof handler === 'string') {
            func = yield this._getCallback(plugin.name, handler);
          }

          if (func === null) {
            return;
          }

          this._consolidateHandlers(eventName, func);

          this.on(eventName, func);
        } catch (error) {
          _colorLogger2.default.e(error.message);
        }
      });

      function _processHandler(_x13, _x14, _x15) {
        return _ref9.apply(this, arguments);
      }

      return _processHandler;
    })()

    /**
     * This removes existing matching handler for an event if necessary.
     *
     * @param {String}   eventName event name
     * @param {Function} handler   the handler
     *
     * @returns {undefined}
     *
     * @private
     */

  }, {
    key: '_consolidateHandlers',
    value: function _consolidateHandlers(eventName, handler) {
      if (this.hasHandler(eventName, handler) === true && this._override === true) {
        this.removeHandler(eventName, handler);
      }
    }

    /**
     * Get callback for plugin service.
     *
     * @param {Object}   pluginName the plugin
     * @param {String}   funcName   the name of the event
     *
     * @returns {Function} the handler
     *
     * @private
     */

  }, {
    key: '_getCallback',
    value: (() => {
      var _ref10 = _asyncToGenerator(function* (pluginName, funcName) {
        try {
          const pg = require(`${ pluginName }`); // eslint-disable-line global-require

          return pg[funcName];
        } catch (error) {
          _colorLogger2.default.e(`Unsupported action '${ funcName }'`, error);
        }

        return null;
      });

      function _getCallback(_x16, _x17) {
        return _ref10.apply(this, arguments);
      }

      return _getCallback;
    })()
  }, {
    key: 'debug',
    get: function get() {
      return this._debug;
    }

    /**
     * Get logger.
     *
     * @returns {Object} the logger instance
     */

  }, {
    key: 'log',
    get: function get() {
      return _colorLogger2.default;
    }
  }]);

  return PluginManager;
}(_events.EventEmitter);

exports.default = PluginManager;